import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Carrito } from './entity/carrito.entity';
import { Modulo } from '../modulo/entity/modulo.entity';
import { InstitucionTipoCarrito } from '../institucion-tipo-carrito/entity/institucion-tipo-carrito.entity';
import { ModuloService } from '../modulo/modulo.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private moduloService: ModuloService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
  ) {}

  // Eliminar carrito (argumento) de este proceso. Buscar cuantos carritos tiene el modulo de ese tipo carrito y automaticamente generar carrito
  async create(id_tipo_carrito: number, id_modulo: number) {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findByIdTipoCarito(
        id_tipo_carrito,
      );
    const nuevoCarrito = this.repository.create({
      modulo,
      tipoCarrito,
    });

    return this.repository
      .findOne({ modulo, tipoCarrito })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException('Este carrito ya existe en este modulo');
        return this.repository.save(nuevoCarrito);
      })
      .then(() => ({
        message: 'Se creo correctamente el carrito en el modulo.',
      }));
  }

  findAll() {
    return this.repository.find();
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este carrito');
      return carrito;
    });
  }

  // Toma de ejemplo la funcion que esta en /src/usuario/usuario.service.ts
  async findByIdModulo(filtros: {
    pagina?: string,
    id_modulo?: string,
    id_tipo_carrito?: string,
    activo?: string,
  }) {
      const busqueda: {
        modulo?: Modulo;
        institucionTipoCarrito?: InstitucionTipoCarrito;
        activo?: string;
      } = {}

      const modulo = filtros.id_modulo
        ? await this.moduloService.findById(Number(filtros.id_modulo))
        : null;
      const institucionTipoCarrito = filtros.id_tipo_carrito
        ? await this.institucionTipoCarritoService.findById(Number(filtros.id_tipo_carrito))
        : null
      
      if (filtros.activo) busqueda.activo = filtros.activo;
      if (modulo) busqueda.modulo = modulo;
      if (institucionTipoCarrito) busqueda.institucionTipoCarrito = institucionTipoCarrito;
      //falta página

      return this.repository.find(busqueda);
  }

  // re hacer
  async update(attrs: Partial<Carrito>) {
    const carrito = await this.findById(attrs.id_carrito);

    return this.repository
      .findOne({ carrito: attrs.carrito, modulo: carrito.modulo })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException('Ya existe un carrito con este número.');
        Object.assign(carrito, attrs);
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del carrito.',
      }));
  }
}
