import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Operador } from './entity/operador.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
import { InstitucionService } from '../institucion/institucion.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';

@Injectable()
export class OperadorService {
  constructor(
    @InjectRepository(Operador) private repository: Repository<Operador>,
    private bcryptService: BcryptService,
    private institucionService: InstitucionService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async create(id_institucion: number, operador: string, password: string) {
    const institucion = await this.institucionService.findById(id_institucion);
    const tipoUsuario = await this.tipoUsuarioService.findById(4);

    return this.repository
      .findOne({ operador, institucion })
      .then(async (existeOperador) => {
        if (existeOperador)
          throw new ConflictException(
            'Ya existe un operador con ese nombre, intente de nuevo.',
          );
        password = this.bcryptService.encriptar(password);
        return this.repository.save(
          this.repository.create({
            institucion,
            operador,
            password,
            tipoUsuario,
          }),
        );
      })
      .then((_) => ({ message: 'Se creo correctamente al operador.' }));
  }

  async findAll(filtros: {
    pagina: string;
    id_institucion?: string;
    operador?: string;
  }) {
    const busqueda: { operador?: string; institucion?: Institucion } = {};
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(Number(filtros.id_institucion))
      : null;

    /* buscar like */
    if (filtros.operador) busqueda.operador = filtros.operador;
    if (institucion) busqueda.institucion = institucion;
    /* falta página */
    return this.repository.find(busqueda);
  }

  findById(id_operador: number) {
    return this.repository.findOne({ id_operador }).then((operador) => {
      if (!operador) throw new NotFoundException('No existe este operador');
      return operador;
    });
  }

  findByOperador(id_institucion: number, operador: string) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) => this.repository.findOne({ institucion, operador }))
      .then((operador) => {
        if (!operador) throw new NotFoundException('No existe este operador.');
        return operador;
      });
  }

  update(attrs: Partial<Operador>) {
    return this.findById(attrs.id_operador)
      .then((operador) => {
        if (attrs.password)
          attrs.password = this.bcryptService.encriptar(attrs.password);
        Object.assign(operador, attrs);
        return this.repository.save(operador);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del operador.',
      }));
  }
}
