import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { Carrera } from '../../carrera/entity/carrera.entity';
import { Institucion } from '../../institucion/entity/institucion.entity';
import { Prestamo } from '../../prestamo/entity/prestamo.entity';
import { TipoUsuario } from '../../tipo-usuario/entity/tipo-usuario.entity';

@Entity()
export class Usuario {
  @PrimaryGeneratedColumn()
  id_usuario: number;

  @Column({ type: Boolean, nullable: false, default: false })
  activo: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  multa: boolean;

  @Column({ type: String, nullable: true, length: 60 })
  password: string;

  @Column({ type: String, nullable: true, length: 10 })
  telefono: string;

  @Column({ type: String, nullable: false, length: 10 })
  usuario: string;

  @ManyToOne(() => Carrera, (carrera) => carrera.usuarios)
  @JoinColumn({ name: 'id_carrera' })
  carrera: Carrera;

  @ManyToOne(() => Institucion, (institucion) => institucion.usuarios)
  @JoinColumn({ name: 'id_institucion' })
  institucion: Institucion;

  @ManyToOne(() => TipoUsuario, (tipoUsuario) => tipoUsuario.usuarios, {
    eager: true,
  })
  @JoinColumn({ name: 'id_tipo_usuario' })
  tipoUsuario: TipoUsuario;

  @OneToMany(() => Prestamo, (prestamo) => prestamo.usuario)
  prestamos: Prestamo[];
}
